using System;
using System.Reflection;
using UnityEditor.Experimental.GraphView;
using UnityEngine;
using UnityEngine.UIElements;

namespace ImageProcessingGraph.Editor
{
    public class IPTPort : Port
    {
        public FieldInfo fieldInfo;
        private VisualElement _exposedPropertyContainer;

        public delegate void OnPortConnectedEvent();
        public OnPortConnectedEvent OnPortConnected;
        
        public delegate void OnPortDisconnectedEvent();
        public OnPortDisconnectedEvent OnPortDisconnected;
        
        public VisualElement ExposedPropertyContainer
        {
            set
            {
                _exposedPropertyContainer = value;
            }
            get
            {
                return _exposedPropertyContainer;
            }
        }
        
        protected IPTPort(Orientation portOrientation, Direction portDirection, Capacity portCapacity, Type type) : base(portOrientation, portDirection, portCapacity, type)
        {
            
        }

        public override void Connect(Edge edge)
        {
            base.Connect(edge);
            OnPortConnected?.Invoke();
        }

        public override void Disconnect(Edge edge)
        {
            base.Disconnect(edge);
            OnPortDisconnected?.Invoke();
        }


        private void PublicOnConnected(Port obj)
        {
            throw new NotImplementedException();
        }

        public static IPTPort Create(IEdgeConnectorListener connectorListener, bool isInput, Type type)
        {
            var port = new IPTPort(Orientation.Horizontal, isInput ? Direction.Input : Direction.Output,
                isInput ? Capacity.Single : Capacity.Multi, type)
            {
                m_EdgeConnector = new EdgeConnector<Edge>(connectorListener),
            };
            port.AddManipulator(port.m_EdgeConnector);
            return port;
        }
    }
}