﻿using System.IO;
using ImageProcessingGraph.Editor.Nodes.NodeAttributes;
using Unity.Collections;
using UnityEditor;
using UnityEngine;

namespace ImageProcessingGraph.Editor.Nodes.Import_Nodes
{
    [NodeInfo("Texture Import", "Imports/Import Texture")]
    public class Texture2DImport : BaseImageNode
    {
        [NodeAttributes.Input("")]
        public Texture2D textureImport;
        
        [NodeAttributes.Output("")]
        public ImageData textureOutput;
        [NodeAttributes.Output("File Name")]
        public string fileName;
        [NodeAttributes.Output("File Directory")]
        public string filePath;

        public override void Process()
        {
            if (this.textureImport != null)
            {
                TextureImporter textureImporter = AssetImporter.GetAtPath(AssetDatabase.GetAssetPath(textureImport)) as TextureImporter;
            
                TextureImporterPlatformSettings texset = textureImporter.GetDefaultPlatformTextureSettings();
                texset.format=TextureImporterFormat.RGBA32;
                texset.maxTextureSize=16384;
                textureImporter.SetPlatformTextureSettings(texset);
            
                TextureImporterSettings settings = new TextureImporterSettings();
                textureImporter.ReadTextureSettings(settings);
                settings.readable = true;
                textureImporter.SetTextureSettings(settings);
                EditorUtility.SetDirty(textureImport);
                textureImporter.SaveAndReimport();
                
                
                this.textureOutput = new ImageData(textureImport);
                this.fileName = textureImport.name;
                this.filePath = Path.GetDirectoryName(AssetDatabase.GetAssetPath(textureImport));
            }
            // else
            //     Debug.LogError("UH!");
        }
    }
}