﻿using ImageProcessingGraph.Editor.Nodes.NodeAttributes;
using Unity.Burst;
using Unity.Collections;
using Unity.Jobs;
using UnityEngine;

namespace ImageProcessingGraph.Editor.Nodes.Types.Image.Utilities
{
    public class SingleChannelColor
    {
        [NodeInfoAttribute("Channel Color", "Utility/Channel Color", false)]
        public class SingleColorChannel : BaseImageNode
        {
            [NodeAttributes.Input("Color")] public GreyscaleValue range;

            [NodeAttributes.Input("Width")] public int Width;

            [NodeAttributes.Input("Height")] public int Height;

            [NodeAttributes.Output("Color")] public SplitChannelData OutputColor;

            public override void Process()
            {
                int pixelCount = Width * Height;
    
                NativeArray<byte> outputData = new NativeArray<byte>(pixelCount, Allocator.Persistent);

                CreateSingleColorJob job = new CreateSingleColorJob
                {
                    color32 = range.value,
                    outputData = outputData,
                    width = Width,
                    height = Height
                };

                job.Run();

                OutputColor = new SplitChannelData(outputData.ToArray(), (Width, Height));
            }

            [BurstCompile]
            struct CreateSingleColorJob : IJob
            {
                public int color32;
                [WriteOnly]
                public NativeArray<byte> outputData;
                public int width;
                public int height;

                public void Execute()
                {
                    // More efficient linear write pattern
                    for (int i = 0; i < width*height; i++)
                    {
                        outputData[i] = (byte)color32;
                    }
                }
            }
        }
    }
}
