﻿using ImageProcessingGraph.Editor.Unity_Image_Processing.Scripts.Editor.Windows;
using UnityEngine;
using UnityEngine.UIElements;

namespace ImageProcessingGraph.Editor.Nodes.Types.Image.Utilities.ViewNode
{
    public class ViewTextureNodeEditor : ImageProcessingGraphNodeVisual
    {
        private UnityEngine.UIElements.Image viewableImage;
        private Foldout foldout;
        private VisualElement buttonRow;

        public ViewTextureNodeEditor(BaseImageNode node, ImageProcessingGraphViewWindow window) : base(node, window)
        {
            //Port 0 is Texture2D
            //Port 1 is ImageData

            IPTPort tex2DPort = InputPorts[0] as IPTPort;
            IPTPort imageDataPort = InputPorts[1] as IPTPort;

            tex2DPort.OnPortConnected += () => { imageDataPort.style.display = DisplayStyle.None; };

            tex2DPort.OnPortDisconnected += () => { imageDataPort.style.display = DisplayStyle.Flex; };

            imageDataPort.OnPortConnected += () => { tex2DPort.style.display = DisplayStyle.None; };

            imageDataPort.OnPortDisconnected += () => { tex2DPort.style.display = DisplayStyle.Flex; };

            this.Q("output").style.display = DisplayStyle.None;

            ViewTextureNode viewImageNode = node as ViewTextureNode;

            foldout = new Foldout
            {
                text = "Texture Viewer",
                value = true
            };
            foldout.style.backgroundColor = new Color(63f / 255f, 63f / 255f, 63f / 255f, 205f / 255f);
            foldout.contentContainer.style.justifyContent = Justify.Center;
            Add(foldout);

            viewableImage = new UnityEngine.UIElements.Image
            {
                scaleMode = ScaleMode.ScaleToFit
            };
            viewableImage.style.width = 256;
            viewableImage.style.height = 256;
            viewableImage.style.backgroundColor = Color.black;
            foldout.Add(viewableImage);

            viewImageNode.onImageUpdated += () => { viewableImage.image = viewImageNode.texture; };

            buttonRow = new VisualElement
            {
                style =
                {
                    flexDirection = FlexDirection.Row,
                    justifyContent = Justify.SpaceBetween,
                    alignItems = Align.Center,
                    marginTop = 8,
                    marginBottom = 8,
                }
            };

            string[] channels = { "RGB", "R", "G", "B", "A" };
            foreach (var channel in channels)
            {
                var btn = new Button(() => OnChannelClicked(viewImageNode, channel))
                {
                    text = channel
                };
                btn.style.flexGrow = 1;
                buttonRow.Add(btn);
            }

            foldout.Add(buttonRow);
        }

        private void OnChannelClicked(ViewTextureNode viewNode, string channel)
        {
            switch (channel)
            {
                case "RGB":
                    viewableImage.image = viewNode.cachedRGB;
                    break;
                case "R":
                    viewableImage.image = viewNode.cachedR;
                    break;
                case "G":
                    viewableImage.image = viewNode.cachedG;
                    break;
                case "B":
                    viewableImage.image = viewNode.cachedB;
                    break;
                case "A":
                    viewableImage.image = viewNode.cachedA;
                    break;
                default:
                    Debug.LogError("[ViewTextureNodeEditor] Unknown channel clicked! Did you hack reality again?");
                    break;
            }
        }
    }
}
